/* Test of the histogram routine.
 * Author: Petr Mikulik
 * Version: April 2004
 * License: GNU GPL
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <values.h>
#include "pm_hist.h"

#define VERBOSE 1


/* Generate random integer number from from to to, including these limits.
 */
int
my_random_int ( int from, int to )
{
    return (int)(from + (double)(to - from + 1) * random() / (RAND_MAX+1.0));
}


/******************************************************
 *		main
 ******************************************************/
int
main()
{
    int i;
    double from = 0;
    double to = 40;
    double logbase = 0; // 0 for linear, otherwise logarithmic

    int n = 14; // pocet cisel
    double b;
    double *a = (double*)malloc(n * sizeof(double));
    t_pm_histogram opts;

#if 0
    srandom( (unsigned)time(NULL) );
#else
    srandom( 100 );
#endif

    fprintf(stderr, "\n****** Generating %i values:\n", n);

    for (i=0; i<n; i++) {
	// Random or non-random numbers?
#if 0
	b = my_random_int((int)from, (int)to);
#else
	if (logbase == 0) // Linear spacing?
	    b = from + i * (1.0/2);
	else // Logarithmic spacing?
    	    b = (i==0) ? 1+from : a[i-1]*logbase;
#endif
	a[i] = b;
    }

    // Print generated numbers
#ifdef VERBOSE
    printf("List of generated data:\n");
    for (i=0; i<n; i++) {
	printf("\t%g", a[i]);
#if 1
	printf("%s", ((i % 5 == 4) ? "\n" : "\t"));
#else
	printf("\n");
#endif
    }
    printf("\nEnd of list of generated data.\n");
#endif // VERBOSE


    // Set histograming options
    opts.autoscale = 1;
    opts.zfrom = 0;
    opts.zto = n-1;
    opts.bins = n;
    opts.logbase = logbase;

    printf("*** Histogram properties:  from=%g  to=%g  bins=%i  logbase=%g\n", opts.zfrom, opts.zto, opts.bins, opts.logbase);
    // Run the histogram
    printf("*** Starting the histogram:\n");
    double *x = (double*)malloc(opts.bins * sizeof(double));
    unsigned long *h = (unsigned long*)malloc(opts.bins * sizeof(unsigned long));
#if 1
    int res = pm_histogram(x, h, a, n, opts);
#else
    for (i=0; i<opts.bins; i++) x[i] = i;
    int res = pm_histogram(NULL, h, a, n, opts, x);
#endif
    if (res) {
	printf("Call to histogram failed with this reason: %i\n", res);
	exit(res);
    }

    // Print it to screen
#ifdef VERBOSE
    printf("*** The histogram:\n");
    for (i=0; i<opts.bins; i++) {
	if (opts.logbase)
	    printf("\t%.3g\t%lu\n", x[i], h[i]);
	else
	    printf("\t%.3f\t%lu\n", x[i], h[i]);
    }
#endif

    fprintf(stderr, "\nDone.\n");
    return 0;
}


// eof pmhist_test.cpp
